%% Autonomous system identification from data with missing values (figure 1)

clear all, close all

np = 50; ng = 20; N = 100;
r  = 2;  s  = linspace(0, 0.1, 7); 

load ica18b_sys0; % sys0 = drss(r);
p0 = impulse(sys0, np); p0(1) = []; 
b0 = p0(1:ng); 

ei = @(ph) norm(p0(1:ng) - ph(1:ng)) / norm(p0(1:ng));
ep = @(ph) norm(p0(1+ng:np) - ph(1+ng:np)) / norm(p0(1+ng:np));

for i = 1:7
    i = i
    for j = 1:N
        bt = randn(ng, 1);
        b  = b0 + s(i) * bt / norm(bt) * norm(b0);
        p  = [b; NaN * ones(np - ng, 1)];
        [sysh, info, ph] = ident(p, 0, r);
        Ei(j, i) = ei(p); Eih(j, i) = ei(ph); % Ep(j, i) = ep(ph);
    end
end

plot(s, mean(Ei) , '-k'), hold on
plot(s, mean(Eih), '--b')
% plot(s, mean(Ep)  / (np - ng), '-.b')
print_fig('ica18b-f1')

%% Comparison with the l_1 -norm regularization method

clear all, close all, warning off

T  = 100; Tm = 14; k = 2; n = 2 * k; s = 0.05; N = 100;

D  = idct(eye(T));
x0 = rand(k, 1);
f  = sort(randperm(T, k)', 1, 'descend');
y0  = D(:, f) * x0;
ym0 = y0(1:Tm);
ymt = randn(Tm, 1);
ym  = ym0 + s * ymt / norm(ymt) * norm(ym0);

[sys0, info0]     = ident(ym0, 0, n); opt = []; % opt.sys0 = sys0;
[sysh, info, ymh] = ident(ym,  0, n, opt);

fh_slra = round(imag(log(eig(sysh))) * T / pi); % frequencies
fh_slra(fh_slra < 0) = []; % filter the negative frequencies
fh_slra = unique(fh_slra + 1); % zero frequency has index 1

for i = 1:N
    i = i
    method = 'l1';
    if strcmp(method, 'l1')
        cvx_begin quiet
        variable x(T)
        minimize( norm(ym - D(1:Tm, :) * x) );
        subject to 
            norm(x, 1) < 1.1 * norm(x0)
            cvx_end
    elseif strcmp(method, 'opm')
        x = omp(D(1:Tm, :), ym);
    elseif strcmp(method, 'csrec')
        sol = CSRec_SP(n, D(1:Tm, :), ym); x = sol.x_hat;
    elseif strcmp(method, 'stls')
        x = STLS_Frame_Sparse(ym, D(1:Tm, :), n, 3, 20, 5, 1);
    end
    [x_max, ind] = sort(x, 1, 'descend'); 
    fh = sort(ind(1:k), 'descend');
    %[{'true' 'slra' method}; num2cell(sort([f fh_slra fh]))]
    C(i) = length(intersect(f, fh));
    C_slra(i) = length(intersect(f, fh_slra));
end

[sum(C) sum(C_slra)] / N / k

